/*
* DEMO.C
* Demonstrates sending data to DPLOT via DDEML functions.
* Two methods are given... a single XTYP_POKE transaction or
* multiple XTYP_EXECUTE transactions.
* The XTYP_POKE method is considerably faster, but requires
* more available memory and requires that all data be known
* prior to sending the first (and only) transaction.
* The XTYP_EXECUTE method is better suited to real-time
* applications or to relatively small data sets.
*
* To build this application using the WATCOM C compiler:
* 16-bit:
* wcc demo.c -i=%WATCOM%\h;%WATCOM%\h\win -zq -otexan -bt=windows -ml
* wcc plot.c -i=%WATCOM%\h;%WATCOM%\h\win -zq -otexan -bt=windows -ml
* wrc demo.rc -bt=windows -i=%WATCOM%\h;%WATCOM%\h\win -r -fo=demo.res
* wlink sys windows libr ddeml.lib,shell.lib fil demo.obj,plot.obj
* wrc demo.res -s0 demo.exe
*
* 32-bit:
* wcc386 demo.c -i=%WATCOM%\h;%WATCOM%\h\nt -bt=nt -mf
* wcc386 plot.c -i=%WATCOM%\h;%WATCOM%\h\nt -bt=nt -mf
* wrc demo.rc -bt=nt -i=%WATCOM%\h;%WATCOM%\h\nt -r -fo=demo.res
* wlink sys win95 fil demo.obj,plot.obj
* wrc demo.res demo.exe
*/
#define INCLUDE_DDEML_H
#include <windows.h>
#include "demo.h"
#include "dplot.h"
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <malloc.h>
#include <math.h>

const float PI=3.14159265358979224f;

DWORD	idInst;
HCONV	hConvDPLOT;
float	A;
float	StartX;
float	EndX;
UINT	NumPoints;
UINT	NX, NY;

char	PlotCaption[64];		// This string is used to name each plot when it is produced.
								// The caption may later be used to retrieve a bitmap of the plot.
								// This falls apart if more than one plot has the same caption.
int		cxBitmap, cyBitmap;
int 	UseDPlotJr;
HBITMAP	hbmp;

extern HWND StartDPLOT(int Jr);
extern HDDEDATA CALLBACK DdeCallback(UINT,UINT,HCONV,HSZ,HSZ,HDDEDATA,DWORD,DWORD);

typedef struct tagXYZ
{
	float x;
	float y;
	float z;
} XYZ;

HBITMAP CopyBitmap(HBITMAP hbmp)
{
	BITMAP	bm;
	HBITMAP	hbmpNew, hbmpPrev1, hbmpPrev2;
	HDC		hdc, hdcMem1, hdcMem2;

	if(!hbmp) return (HBITMAP)NULL;
	if(!GetObject(hbmp,sizeof(BITMAP),&bm)) return (HBITMAP)NULL;
	hdc = GetDC(NULL);
	hbmpNew = CreateCompatibleBitmap(hdc,bm.bmWidth,bm.bmHeight);
	if(!hbmpNew) return (HBITMAP)NULL;
	hdcMem1 = CreateCompatibleDC(hdc);
	hdcMem2 = CreateCompatibleDC(hdc);
	hbmpPrev1 = SelectObject(hdcMem1,hbmp);
	hbmpPrev2 = SelectObject(hdcMem2,hbmpNew);
	BitBlt(hdcMem2,0,0,bm.bmWidth,bm.bmHeight,hdcMem1,0,0,SRCCOPY);
	SelectObject(hdcMem1,hbmpPrev1);
	SelectObject(hdcMem2,hbmpPrev2);
	DeleteDC(hdcMem2);
	DeleteDC(hdcMem1);
	ReleaseDC(NULL,hdc);
	return hbmpNew;
}

void DrawBitmap(HDC hdc,int xleft,int ytop,HBITMAP hBitmap)
{
	BITMAP		bm;
	HBITMAP		hbmpOld;
	HDC			hdcMem;

	hdcMem = CreateCompatibleDC(hdc);
	if(hdcMem)
	{
		hbmpOld = SelectObject(hdcMem,hBitmap);
		GetObject(hBitmap,sizeof(BITMAP),&bm);
		BitBlt(hdc,xleft,ytop,bm.bmWidth,bm.bmHeight,hdcMem,0,0,SRCCOPY);
		SelectObject(hdcMem,hbmpOld);
		DeleteDC(hdcMem);
	}
}

HBITMAP	GetDPlotBitmap(int cxBitmap, int cyBitmap)
{
	HBITMAP		hbmpNew, hbmpTemp;
	HDDEDATA	hDdeData;
	HSZ			hszPic;
	LPBYTE		lpData;
	char		szExec[256];

	hbmpNew = (HBITMAP)NULL;

	// Set desired size of bitmap we'll ask DPlot for:
	sprintf(szExec,"[SetBitmapDims(%d,%d)]",cxBitmap,cyBitmap);
	DdeClientTransaction( szExec, strlen(szExec)+1,
		hConvDPLOT,(HSZ)NULL, CF_TEXT, XTYP_EXECUTE, 1000, NULL);

	hszPic = DdeCreateStringHandle(idInst,"Picture",CP_WINANSI);
	hDdeData = DdeClientTransaction( (LPBYTE)NULL,0,hConvDPLOT,
		hszPic,CF_BITMAP,XTYP_REQUEST,1000,NULL);
	if(hDdeData)
	{
		lpData = DdeAccessData(hDdeData,NULL);
		memcpy(&hbmpTemp,lpData,sizeof(HBITMAP));
		if(hbmpTemp)
		{
			hbmpNew = CopyBitmap(hbmpTemp);
			DeleteObject(hbmpTemp);
		}
		DdeUnaccessData(hDdeData);
		DdeFreeDataHandle(hDdeData);
	}
	DdeFreeStringHandle(idInst,hszPic);
	return hbmpNew;
}

void DrawBitmapToCtrl(HWND hwnd, int CtrlID)
{
	HDC		hdc;
	RECT	rc;

	GetWindowRect(GetDlgItem(hwnd,CtrlID),&rc);
	MapWindowPoints(NULL,hwnd,(LPPOINT)&rc,2);
	hdc = GetDC(hwnd);
	DrawBitmap(hdc,rc.left+2,rc.top+2,hbmp);
	ReleaseDC(hwnd,hdc);
}

BOOL CALLBACK PlotOptionsDlgProc( HWND hwnd, unsigned msg,
								 UINT wparam, LONG lparam )
{
	
	const DWORD 	TimeOut=1000;
	BOOL		Result;
	char		szExec[512];
	char		szText[256];
	DPLOT		DPlot;
	float		dX, dY;
	float		X;
	float		*Y;
	float		*Z, *pZ;
	XYZ 		*Node;
	float		XP, YP;
	float		temp;
	HDDEDATA	hDdeData;
	HSZ 		hszItem, hszService, hszTopic;
	DWORD		cbAddData;
	UINT		cfFormat;
	DWORD		dwOffset;
	DWORD		dwSize;
	DWORD		i, j;
	DWORD		t1, t2;
	RECT		rc;
#define MAX_ADD 32768
#define MAX_3DADD 32760 			// divisible by sizeof(XYZ)
	
	lparam = lparam;					/* turn off warning */
	
	switch( msg ) {
	case WM_INITDIALOG:
		idInst = 0;
		DdeInitialize(&idInst,
			(PFNCALLBACK)MakeProcInstance((FARPROC)DdeCallback,(HINSTANCE)GetWindowLong(hwnd,GWL_HINSTANCE)), 
			CBF_FAIL_ALLSVRXACTIONS,
			0);
		
		A = 0.5;
		StartX = 0.;
		EndX = 10.;
		NumPoints = 1001;
		UseDPlotJr = 0;
		sprintf(szText, "%.5g", A);
		SetDlgItemText(hwnd,DLG_A,szText);
		sprintf(szText, "%.5g", StartX);
		SetDlgItemText(hwnd,DLG_STARTX,szText);
		sprintf(szText, "%.5g", EndX);
		SetDlgItemText(hwnd,DLG_ENDX,szText);
		sprintf(szText, "%d", NumPoints);
		SetDlgItemText(hwnd,DLG_NUMPOINTS,szText);
		SetDlgItemText(hwnd,DLG_NUM3DRAND,"100");
		SetDlgItemText(hwnd,DLG_NX,"50");
		SetDlgItemText(hwnd,DLG_NY,"50");

		// Desired size of bitmap retrieved from DPlot:
		GetClientRect(GetDlgItem(hwnd,DLG_BMPFRAME),&rc);
		cxBitmap = rc.right-rc.left-1;
		cyBitmap = rc.bottom-rc.top-1;
		return TRUE;
		
	case WM_CLOSE:
		EndDialog(hwnd,TRUE);
		return TRUE;
		
	case WM_COMMAND:
		switch( LOWORD( wparam ) ) {
		case DLG_DPLOTJR:
			UseDPlotJr = IsDlgButtonChecked(hwnd,DLG_DPLOTJR);
			EnableWindow(GetDlgItem(hwnd,DLG_NOTE),1-UseDPlotJr);
			return TRUE;
		case DLG_GETBMP:
			if(!strlen(PlotCaption))
			{
				MessageBeep(MB_ICONEXCLAMATION);
				return TRUE;
			}
			hszService = DdeCreateStringHandle( idInst, "DPLOT", CP_WINANSI );
			hszTopic   = DdeCreateStringHandle( idInst, PlotCaption, CP_WINANSI );
			hConvDPLOT = DdeConnect( idInst, hszService, hszTopic, NULL );
			DdeFreeStringHandle(idInst,hszService);
			DdeFreeStringHandle(idInst,hszTopic);
			if( hConvDPLOT )
			{
				if(hbmp)
				{
					DeleteObject(hbmp);
					hbmp = (HBITMAP)NULL;
				}
				hbmp = GetDPlotBitmap(cxBitmap,cyBitmap);
				if(hbmp) DrawBitmapToCtrl(hwnd,DLG_BMPFRAME);
				DdeDisconnect(hConvDPLOT);
			}
			else
				MessageBeep(MB_ICONEXCLAMATION);
			return TRUE;

		case DLG_PLOTFAST:
			/* Plot entire data set all at once. This method is considerably
			   faster than one point at a time (see below), but requires
			   a little more work and a lot more memory. */
			GetDlgItemText(hwnd,DLG_A,szText,sizeof(szText));
			A = (float)atof(szText);
			GetDlgItemText(hwnd,DLG_STARTX,szText,sizeof(szText));
			StartX = (float)atof(szText);
			GetDlgItemText(hwnd,DLG_ENDX,szText,sizeof(szText));
			EndX = (float)atof(szText);
			GetDlgItemText(hwnd,DLG_NUMPOINTS,szText,sizeof(szText));
			NumPoints = atoi(szText);
			
			if( NumPoints < 2 ) {
				MessageBox(hwnd,"The number of points must be greater than 1.","Demo",MB_OK);
				return TRUE;
			}
			
			if( !StartDPLOT(UseDPlotJr) ) {
				MessageBox(hwnd,"Cannot find DPLOT.","Demo",MB_OK);
				return TRUE;
			}
			
			t1 = GetTickCount();
			hszService = DdeCreateStringHandle( idInst, "DPLOT", CP_WINANSI );
			hszTopic   = DdeCreateStringHandle( idInst, "XYPlot", CP_WINANSI );
			hConvDPLOT = DdeConnect( idInst, hszService, hszTopic, NULL );
			DdeFreeStringHandle(idInst,hszService);
			DdeFreeStringHandle(idInst,hszTopic);
			if( !hConvDPLOT ) {
				// Connection not established... we must
				// have started an old, pre-DDE version of DPLOT
				MessageBox(hwnd,"Cannot initiate DPLOT server!","Demo",
					MB_OK | MB_ICONEXCLAMATION);
				return TRUE;
			}
			
			if( NULL == (Y = (float *)calloc( 2*NumPoints, sizeof(float)))) {
				DdeDisconnect(hConvDPLOT);
				MessageBox(hwnd,"Memory allocation failure.","Demo",
					MB_OK | MB_ICONEXCLAMATION);
				return TRUE;
			}
			dX = (EndX-StartX)/(NumPoints-1);
			for( j=0; j < NumPoints; j++ ) {
				X			   = StartX + j*dX;
				Y[j]		   = (float)sin(A*PI*X);
				Y[j+NumPoints] = (float)cos(A*PI*X);
			}
			
			/* Initialize DPLOT structure to 0's: */
			memset(&DPlot,0,sizeof(DPlot));
			
			/* Fill in members of DPLOT structure */
			DPlot.Version	  = DPLOT_DDE_VERSION;
			DPlot.hwnd		  = (DWORD)hwnd;
			DPlot.DataFormat  = DATA_DXY;
			DPlot.MaxCurves   = 2;			   // Set to larger number to allow user to easily add 1 or more curves.
			DPlot.MaxPoints   = NumPoints;
			DPlot.NumCurves   = 2;
			DPlot.Scale 	  = SCALE_LINEARX_LINEARY;
			DPlot.LegendX	  = 0.05f;
			DPlot.LegendY	  = 0.05f;
			DPlot.NP[0] 	  = NumPoints;
			DPlot.NP[1] 	  = NumPoints;			// Not needed, since NP[1]=NP[0] is implied by DATA_DXY
			strcpy(DPlot.Legend[0],"Legend");
			sprintf(DPlot.Legend[1],"sin(%.5g{\\sp}X)",A);
			sprintf(DPlot.Legend[2],"cos(%.5g{\\sp}X)",A);
			DPlot.LineType[0] = LINESTYLE_SOLID;
			DPlot.LineType[1] = LINESTYLE_LONGDASH;
			strcpy(DPlot.Title[0],"DPLOT DDE Demo");
			strcpy(DPlot.XAxis,"X");
			strcpy(DPlot.YAxis,"Y");
			
			strcpy(PlotCaption,"Demo - Plot Fast Example");
			sprintf(szExec,"[TextNote(\""
				"This note is not a part of the standard DPLOT DDE\r\n"
				"structure. It was added to the DDE data object following\r\n"
				"the DPLOT structure and the data values, and the\r\n"
				"SizeofExtraInfo member of the DPLOT structure was set to\r\n"
				"the size of the execute command used to generate this note.\r\n"
				"You could accomplish the same result by using a second call\r\n"
				"to DdeClientTransaction, but not without causing DPlot to\r\n"
				"update the screen between calls.\",0.45,0.75,1,1,1,8)]"
				"[Caption(\"%s\")]",PlotCaption);
			DPlot.SizeofExtraInfo = strlen(szExec)+1;
			
			/* Find total memory requirements for DDE data object, based
			on data format and number of points.
			There are 3 methods for sending X,Y data to DPLOT. Each
			method uses a block of memory allocated with DdeCreateDataHandle.
			The DPLOT structure is copied to this memory block, followed by
			the X and Y data:
			DATA_XYXY can be used in all cases. The DPLOT structure is followed
			by the X values for the 1st curve, 2nd curve, etc. and then
			by the Y values for the 1st curve, 2nd curve, etc.
			DATA_XYYY may be used if all curves share the same X values. In this
			case, the DPLOT structure is followed by the X array, then
			the Y arrays for the 1st curve, 2nd curve, etc.
			DATA_DXY  may be used if all curves share the same X values AND X
			has a constant spacing. In this case, the DPLOT structure
			is followed by the X spacing, the inital X value, then
			the Y arrays.
			In this demo we are using DATA_DXY since it has the smallest memory
			requirement and works for our data set. 						  */
			
			dwSize = sizeof(DPlot);
			switch( DPlot.DataFormat ) {
			case DATA_XYXY:
				for(i=0; i < DPlot.NumCurves; i++ ) {
					dwSize += DPlot.NP[i]*2*sizeof(float);
				}
				break;
			case DATA_DXY:
				dwSize += (2+DPlot.NumCurves*DPlot.NP[0])*sizeof(float);
				break;
			case DATA_XYYY:
				dwSize += (1+DPlot.NumCurves)*DPlot.NP[0]*sizeof(float);
				break;
			}
			dwSize += DPlot.SizeofExtraInfo;
			hszItem = DdeCreateStringHandle(idInst,"Data",CP_WINANSI);
			cfFormat = RegisterClipboardFormat("DPLOT");
			/* Create an empty memory block the size of the total required memory. This avoids
			forcing memory reallocation for each call to DdeAddData, but is likely to fail
			for very large (>1 million points) data sets. If this is the case, create the
			data handle with, for example, sizeof(DPlot) instead of dwSize. In this case
			each subsequent call to DdeAddData will cause a memory reallocation and
			slow this process down quite a bit... I'm not sure why this works since the
			total memory requirement is the same, but it does. */
			hDdeData = DdeCreateDataHandle( idInst, NULL, 0, dwSize, hszItem, cfFormat, 0 );
			if( !hDdeData ) {
			/* Error codes are defined in DDEML.H. An error at this point is probably
				0x4008 (DMLERR_MEMORY_ERROR), indicating insufficient memory. */
				sprintf(szText,"Error code for DdeCreateDataHandle=%#x",DdeGetLastError(idInst));
				DdeFreeStringHandle(idInst,hszItem);
				DdeDisconnect(hConvDPLOT);
				free(Y);
				MessageBox(hwnd,szText,"Demo",0);
				return TRUE;
			}
			
			// Copy DPLOT structure to hDdeData
			dwOffset  = 0;
			cbAddData = sizeof(DPlot);
			hDdeData  = DdeAddData(hDdeData,(LPBYTE)&DPlot,cbAddData,dwOffset);
			dwOffset  += cbAddData;

			// Copy dX and StartX
			cbAddData = sizeof(float);
			hDdeData = DdeAddData(hDdeData,(LPBYTE)&dX,cbAddData,dwOffset);
			dwOffset += cbAddData;
			hDdeData = DdeAddData(hDdeData,(LPBYTE)&StartX,cbAddData,dwOffset);
			dwOffset += cbAddData;
			
		   /* Copy Y values. Some versions of DDEML exhibit problems when adding very
			  large chunks of data. Here we have set the maximum to MAX_ADD 
			  (defined above) and loop until Y arrays are exhausted. */
			dwSize = DPlot.NumCurves*DPlot.NP[0]*sizeof(float);
			j = 0;
			while( dwSize > 0 ) {
				cbAddData = min(MAX_ADD,dwSize);
				hDdeData	= DdeAddData(hDdeData,(LPBYTE)&Y[j],cbAddData,dwOffset);
				if( !hDdeData ) {
					sprintf(szText,"Error code for DdeAddData=%#x",DdeGetLastError(idInst));
					DdeFreeStringHandle(idInst,hszItem);
					DdeDisconnect(hConvDPLOT);
					free(Y);
					MessageBox(hwnd,szText,"DDE Error",MB_OK | MB_ICONEXCLAMATION);
					return TRUE;
				}
				dwOffset += cbAddData;
				dwSize -= cbAddData;
				j += cbAddData/sizeof(float);
			}

			// Copy [TextNote()] command
			cbAddData = DPlot.SizeofExtraInfo;
			hDdeData  = DdeAddData(hDdeData,szExec,cbAddData,dwOffset);
			dwOffset  += cbAddData;
			
			if( DdeClientTransaction( (LPBYTE)hDdeData,
				-1,
				hConvDPLOT,
				hszItem,
				cfFormat,
				XTYP_POKE,
				TimeOut,
				NULL		   ) == NULL ) {
				sprintf(szText,"Error code for DdeClientTransaction=%#x",DdeGetLastError(idInst));
				MessageBox(hwnd,szText,"DDE Error",MB_OK | MB_ICONEXCLAMATION);
			}
			DdeFreeStringHandle(idInst,hszItem);
			DdeDisconnect(hConvDPLOT);
			free(Y);
			t2 = GetTickCount();
			sprintf(szText,"Elapsed time = %d milliseconds",t2-t1);
			SetDlgItemText(hwnd,DLG_TIME,szText);
			return TRUE;
			
		case DLG_PLOTSLOW:
			/* Plot data one point at a time using XTYP_EXECUTE command strings.
			This method is very slow, particularly for large data sets, but
			doesn't require as much memory as sending entire data sets with
			XTYP_POKE.
			A complete list of command strings is found in the DPLOT Help
			file... search for XTYP_EXECUTE.
			NOTE: Very little error checking is done here. A return value of
			DMLERR_NOTPROCESSED from DdeClientTransaction generally indicates
			that the command string was not recognized by DPLOT. */
			GetDlgItemText(hwnd,DLG_A,szText,sizeof(szText));
			A = (float)atof(szText);
			GetDlgItemText(hwnd,DLG_STARTX,szText,sizeof(szText));
			StartX = (float)atof(szText);
			GetDlgItemText(hwnd,DLG_ENDX,szText,sizeof(szText));
			EndX = (float)atof(szText);
			GetDlgItemText(hwnd,DLG_NUMPOINTS,szText,sizeof(szText));
			NumPoints = atoi(szText);
			
			if( NumPoints < 2 ) {
				MessageBox(hwnd,"The number of points must be greater than 1.","Demo",MB_OK);
				return TRUE;
			}
			
			if( !StartDPLOT(UseDPlotJr) ) {
				MessageBox(hwnd,"Cannot find DPLOT.","Demo",MB_OK);
				return TRUE;
			}
			
			t1 = GetTickCount();
			hszService = DdeCreateStringHandle( idInst, "DPLOT", CP_WINANSI );
			hszTopic   = DdeCreateStringHandle( idInst, "System", CP_WINANSI );
			hConvDPLOT = DdeConnect( idInst, hszService, hszTopic, NULL );
			DdeFreeStringHandle(idInst,hszService);
			DdeFreeStringHandle(idInst,hszTopic);
			if( !hConvDPLOT ) {
				// Connection not established... we must
				// have started an old, pre-DDE version of DPLOT
				MessageBox(hwnd,"Cannot initiate DPLOT server!","Demo",
					MB_OK | MB_ICONEXCLAMATION);
				return TRUE;
			}
			
			/* Insure DPLOT is not minimized and open a
			new document window */
			strcpy(szText,"[AppRestore()][FileNew()]");
			DdeClientTransaction( szText,
				strlen(szText)+1,
				hConvDPLOT,
				(HSZ) NULL,
				CF_TEXT,
				XTYP_EXECUTE,
				TimeOut,
				NULL		   );
			
			/* Set minimum array sizes */
			sprintf(szText,"[FileArrays(2,%d)]", NumPoints);
			DdeClientTransaction( szText,
				strlen(szText)+1,
				hConvDPLOT,
				(HSZ) NULL,
				CF_TEXT,
				XTYP_EXECUTE,
				TimeOut,
				NULL		   );
			
			strcpy(szText,"[AutoSize()][AutoScale()]");
			DdeClientTransaction( szText,
				strlen(szText)+1,
				hConvDPLOT,
				(HSZ) NULL,
				CF_TEXT,
				XTYP_EXECUTE,
				TimeOut,
				NULL		   );
			
			/* Setting line widths to 0 improves performance */
			strcpy(szText,"[LineWidth(1,0)][LineWidth(2,0)]");
			DdeClientTransaction( szText,
				strlen(szText)+1,
				hConvDPLOT,
				(HSZ) NULL,
				CF_TEXT,
				XTYP_EXECUTE,
				TimeOut,
				NULL		   );
			
			
				/*		   Use PostponeRedraw to delay rendering of the data.
				This generally results in an improvement in
				performance and prevents DPLOT from re-sizing the
				plot many times. However, you may WANT to see
				the data as it is sent.
				strcpy(szText,"[PostponeRedraw()]");
				DdeClientTransaction( szText,
				strlen(szText)+1,
				hConvDPLOT,
				(HSZ) NULL,
				CF_TEXT,
				XTYP_EXECUTE,
				TimeOut,
				NULL		   );
			*/
			
			/*		   Alternative to using PostponeRedraw, you can use
			the ManualScale command to prevent the display
			from being redrawn hundreds of times. When used
			in conjunction with a series of XY() commands, the
			ManualScale command causes DPLOT to use a sliding
			window to view the last points sent via XY().
			The following example uses a window width equal to
			1/4 of the range in X */
			sprintf(szText,"[ManualScale(%g,-1.5,%g,1.5)]",StartX,StartX+(EndX-StartX)/4.);
			DdeClientTransaction( szText,
				strlen(szText)+1,
				hConvDPLOT,
				(HSZ) NULL,
				CF_TEXT,
				XTYP_EXECUTE,
				TimeOut,
				NULL		   );
			
			dX = (EndX-StartX)/(NumPoints-1);
			Result = TRUE;
			j = 0;
			while( j < NumPoints && Result ) {
				XP = StartX + j*dX;
				YP = (float)sin(A*PI*XP);
				sprintf(szText,"[SelectCurve(1)][XY(%g,%g)]",XP,YP);
				Result = (BOOL)DdeClientTransaction( szText,
					strlen(szText)+1,
					hConvDPLOT,
					(HSZ) NULL,
					CF_TEXT,
					XTYP_EXECUTE,
					TimeOut,
					NULL		   );
				YP = (float)cos(A*PI*XP);
				sprintf(szText,"[SelectCurve(2)][XY(%.5g,%.5g)]",XP,YP);
				Result = (BOOL)DdeClientTransaction( szText,
					strlen(szText)+1,
					hConvDPLOT,
					(HSZ) NULL,
					CF_TEXT,
					XTYP_EXECUTE,
					TimeOut,
					NULL		   );
				j++;
			}
			
			strcpy(szText,"[Title1(\"DPLOT DDE Demo\")]");
			DdeClientTransaction( szText,
				strlen(szText)+1,
				hConvDPLOT,
				(HSZ) NULL,
				CF_TEXT,
				XTYP_EXECUTE,
				TimeOut,
				NULL		   );
			
				/* All of the changes to the plot that we have just
				made cause DPLOT to set a flag indicating that the
				plot has been edited... so if we now close the plot
				DPLOT prompts us to save it first. We can clear this
			flag at this point with... */
			strcpy(szText,"[ClearEditFlag()]");
			DdeClientTransaction( szText,
				strlen(szText)+1,
				hConvDPLOT,
				(HSZ) NULL,
				CF_TEXT,
				XTYP_EXECUTE,
				TimeOut,
				NULL		   );

			// Set caption
			strcpy(PlotCaption,"Demo - Plot Slow Example");
			sprintf(szText,"[Caption(\"%s\")]",PlotCaption);
			DdeClientTransaction( szText,
				strlen(szText)+1,
				hConvDPLOT,
				(HSZ) NULL,
				CF_TEXT,
				XTYP_EXECUTE,
				TimeOut,
				NULL		   );

			/* If PostponeRedraw() was used, make sure to... */ 			  
			strcpy(szText,"[ViewRedraw()]");
			DdeClientTransaction( szText,
				strlen(szText)+1,
				hConvDPLOT,
				(HSZ) NULL,
				CF_TEXT,
				XTYP_EXECUTE,
				TimeOut,
				NULL		   );

			DdeDisconnect(hConvDPLOT);
			t2 = GetTickCount();
			sprintf(szText,"Elapsed time = %d milliseconds",t2-t1);
			SetDlgItemText(hwnd,DLG_TIME,szText);
			return TRUE;
			
		case DLG_PLOT3DRAND:
			/* Plot random 3D points, generating a contour plot.*/
			GetDlgItemText(hwnd,DLG_NUM3DRAND,szText,sizeof(szText));
			NumPoints = atoi(szText);
			if( NumPoints < 3 ) {
				MessageBox(hwnd,"The number of points must be greater than 2.","Demo",MB_OK);
				return TRUE;
			}
			if( !StartDPLOT(UseDPlotJr) ) {
				MessageBox(hwnd,"Cannot find DPLOT.","Demo",MB_OK);
				return TRUE;
			}
			t1 = GetTickCount();
			hszService = DdeCreateStringHandle( idInst, "DPLOT", CP_WINANSI );
			hszTopic   = DdeCreateStringHandle( idInst, "XYPlot", CP_WINANSI );
			hConvDPLOT = DdeConnect( idInst, hszService, hszTopic, NULL );
			DdeFreeStringHandle(idInst,hszService);
			DdeFreeStringHandle(idInst,hszTopic);
			if( !hConvDPLOT ) {
				// Connection not established... we must
				// have started an old, pre-DDE version of DPLOT
				MessageBox(hwnd,"Cannot initiate DPLOT server!","Demo",
					MB_OK | MB_ICONEXCLAMATION);
				return TRUE;
			}
			/* Generate the data in the range -PI < X < PI, -PI < Y < PI, Z=cos(X)*sin(Y) */
			if( NULL == (Node = (XYZ *)calloc( NumPoints, sizeof(XYZ)))) {
				DdeDisconnect(hConvDPLOT);
				MessageBox(hwnd,"Memory allocation failure.","Demo",
					MB_OK | MB_ICONEXCLAMATION);
				return TRUE;
			}
			for( j=0; j < NumPoints; j++ ) {
  				Node[j].x = -M_PI + 2*M_PI * (float)rand()/(float)RAND_MAX;
  				Node[j].y = -M_PI + 2*M_PI * (float)rand()/(float)RAND_MAX;
				Node[j].z = (float)(cos(Node[j].x) * sin(Node[j].y));
			}
			/* Initialize DPLOT structure to 0's: */
			memset(&DPlot,0,sizeof(DPlot));
			/* Fill in members of DPLOT structure */
			DPlot.Version	 = DPLOT_DDE_VERSION;
			DPlot.hwnd		 = (DWORD)hwnd;
			DPlot.DataFormat  = DATA_3DR;
			DPlot.MaxCurves	 = 1;				   // ALWAYS 1 for 3DR
			DPlot.MaxPoints	 = NumPoints;
			DPlot.NumCurves	 = 1;				   // Ditto
			DPlot.Scale		 = SCALE_LINEARX_LINEARY;
			strcpy(DPlot.Title[0],"DPLOT DDE Demo");
			strcpy(DPlot.XAxis,"X");
			strcpy(DPlot.YAxis,"Y");

			strcpy(PlotCaption,"Demo - 3D Random Points");
			sprintf(szExec,"[Caption(\"%s\")][ContourMethod(1)][ContourLevels(21,-1,1)]",PlotCaption);
			DPlot.SizeofExtraInfo = strlen(szExec)+1;
			
			/* Find total memory requirements for DDE data object, based
			   on data format and number of points.
			   Similar to XY plots, we use a block of memory allocated with 
			   DdeCreateDataHandle. The DPLOT structure is copied to this 
			   memory block, followed by the X,Y,Z triplets. */
			dwSize = sizeof(DPlot) + NumPoints*sizeof(XYZ) + DPlot.SizeofExtraInfo;
			hszItem = DdeCreateStringHandle(idInst,"Data",CP_WINANSI);
			cfFormat = RegisterClipboardFormat("DPLOT");
			hDdeData = DdeCreateDataHandle( idInst, NULL, 0, dwSize, hszItem, cfFormat, 0 );
			if( !hDdeData ) {
			/* Error codes are defined in DDEML.H. An error at this point is probably
			   0x4008 (DMLERR_MEMORY_ERROR), indicating insufficient memory. */
				sprintf(szText,"Error code for DdeCreateDataHandle=%#x",DdeGetLastError(idInst));
				DdeFreeStringHandle(idInst,hszItem);
				DdeDisconnect(hConvDPLOT);
				free(Node);
				MessageBox(hwnd,szText,"Demo",0);
				return TRUE;
			}
			/* Copy DPLOT structure to hDdeData */
			dwOffset  = 0;
			cbAddData = sizeof(DPlot);
			hDdeData  = DdeAddData(hDdeData,(LPBYTE)&DPlot,cbAddData,dwOffset);
			dwOffset  += cbAddData;
			/* Add 3D points. */
			/* Some versions of DDEML exhibit problems when adding very large
			   chunks of data. Here we have set the maximum to MAX_3DADD (defined above)
			   and loop until Node array is exhausted. */
			dwSize = NumPoints*sizeof(XYZ);
			j = 0;
			while( dwSize >	0 ) {
				cbAddData = min(MAX_3DADD,dwSize);
				hDdeData  = DdeAddData(hDdeData,(LPBYTE)&Node[j],cbAddData,dwOffset);
				if( !hDdeData ) {
					sprintf(szText,"Error code for DdeAddData=%#x",DdeGetLastError(idInst));
					DdeFreeStringHandle(idInst,hszItem);
					DdeDisconnect(hConvDPLOT);
					free(Node);
					MessageBox(hwnd,szText,"DDE Error",MB_OK | MB_ICONEXCLAMATION);
					return TRUE;
				}
				dwOffset += cbAddData;
				dwSize -= cbAddData;
				j += cbAddData/sizeof(XYZ);
			}
			cbAddData = DPlot.SizeofExtraInfo;
			hDdeData  = DdeAddData(hDdeData,szExec,cbAddData,dwOffset);
			dwOffset  += cbAddData;

			if( DdeClientTransaction( (LPBYTE)hDdeData,
			   -1,
			   hConvDPLOT,
			   hszItem,
			   cfFormat,
			   XTYP_POKE,
			   TimeOut,
			   NULL 		  ) == NULL ) {
			   sprintf(szText,"Error code for DdeClientTransaction=%#x",DdeGetLastError(idInst));
			   MessageBox(hwnd,szText,"DDE Error",MB_OK | MB_ICONEXCLAMATION);
			}
			DdeFreeStringHandle(idInst,hszItem);
			DdeDisconnect(hConvDPLOT);
			free(Node);
			t2 = GetTickCount();
			sprintf(szText,"Elapsed time = %d milliseconds",t2-t1);
			SetDlgItemText(hwnd,DLG_TIME,szText);
			return TRUE;

		case DLG_PLOT3DGRID:
			/* Plot 3D points on a rectangular grid, generating a contour plot.*/
			GetDlgItemText(hwnd,DLG_NX,szText,sizeof(szText));
			NX = atoi(szText);
			GetDlgItemText(hwnd,DLG_NY,szText,sizeof(szText));
			NY = atoi(szText);

			if( NX < 1 ) {
				MessageBox(hwnd,"NX must be greater than 0.","Demo",MB_OK);
				return TRUE;
			}
			if( NY < 1 ) {
				MessageBox(hwnd,"NY must be greater than 0.","Demo",MB_OK);
				return TRUE;
			}
			if( !StartDPLOT(UseDPlotJr) ) {
				MessageBox(hwnd,"Cannot find DPLOT.","Demo",MB_OK);
				return TRUE;
			}

			t1 = GetTickCount();
			hszService = DdeCreateStringHandle( idInst, "DPLOT", CP_WINANSI );
			hszTopic   = DdeCreateStringHandle( idInst, "XYPlot", CP_WINANSI );
			hConvDPLOT = DdeConnect( idInst, hszService, hszTopic, NULL );
			DdeFreeStringHandle(idInst,hszService);
			DdeFreeStringHandle(idInst,hszTopic);
			if( !hConvDPLOT ) {
			   // Connection not established... we must
			   // have started an old, pre-DDE version of DPLOT
				MessageBox(hwnd,"Cannot initiate DPLOT server!","Demo",
					MB_OK | MB_ICONEXCLAMATION);
				return TRUE;
			}

			/* Generate the data. - NumPoints randomly placed points
			   in the range -PI < X < PI, -PI < Y < PI, Z=cos(X)*sin(Y) */
			if( NULL == (Z = (float *)calloc( (NX+1)*(NY+1), sizeof(float)))) {
				DdeDisconnect(hConvDPLOT);
				MessageBox(hwnd,"Memory allocation failure.","Demo",
				   MB_OK | MB_ICONEXCLAMATION);
				return TRUE;
			}
			dX = 2*M_PI / NX;
			dY = 2*M_PI / NY;
			pZ = Z;
			for(i=0; i<=NX; i++)
			{
				XP = -M_PI + i*dX;
				for(j=0; j<=NY; j++)
				{
					YP = -M_PI + j*dY;
					*pZ = (float)(cos(XP) * sin(YP));
					pZ++;
				}
			}

			/* Initialize DPLOT structure to 0's: */
			memset(&DPlot,0,sizeof(DPlot));

			/* Fill in members of DPLOT structure */
			DPlot.Version	 = DPLOT_DDE_VERSION;
			DPlot.hwnd		 = (DWORD)hwnd;
			DPlot.DataFormat  = DATA_3D;
			DPlot.MaxCurves	 = NX;
			DPlot.MaxPoints	 = NY;
			DPlot.NumCurves	 = 1;
			DPlot.Scale		 = SCALE_LINEARX_LINEARY;
			strcpy(DPlot.Title[0],"DPLOT DDE Demo");
			strcpy(DPlot.XAxis,"X");
			strcpy(DPlot.YAxis,"Y");

			strcpy(PlotCaption,"Demo - 3D Points on a Grid");
			sprintf(szExec,"[Caption(\"%s\")][Contour3D(1)][ContourView(20,30)]"
				"[ContourAxes(1)][ContourGrid(1)][ContourLevels(20,-1,1)]",PlotCaption);
			DPlot.SizeofExtraInfo = strlen(szExec)+1;

			/* Find total memory requirements for DDE data object, based
			   on data format and number of points.
			   Similar to XY plots, we use a block of memory allocated with 
			   DdeCreateDataHandle. The DPLOT structure is copied to this 
			   memory block, followed by extents of the plot, then the Z data. */

			dwSize = sizeof(DPlot) + (4 + (NX+1)*(NY+1))*sizeof(float) + DPlot.SizeofExtraInfo;
			hszItem = DdeCreateStringHandle(idInst,"Data",CP_WINANSI);
			cfFormat = RegisterClipboardFormat("DPLOT");
			hDdeData = DdeCreateDataHandle( idInst, NULL, 0, dwSize, hszItem, cfFormat, 0 );
			if( !hDdeData ) {
			/* Error codes are defined in DDEML.H. An error at this point is probably
			   0x4008 (DMLERR_MEMORY_ERROR), indicating insufficient memory. */
				sprintf(szText,"Error code for DdeCreateDataHandle=%#x",DdeGetLastError(idInst));
				DdeFreeStringHandle(idInst,hszItem);
				DdeDisconnect(hConvDPLOT);
				free(Z);
				MessageBox(hwnd,szText,"Demo",0);
				return TRUE;
			}

			/* Copy DPLOT structure to hDdeData */
			dwOffset  = 0;
			cbAddData = sizeof(DPlot);
			hDdeData  = DdeAddData(hDdeData,(LPBYTE)&DPlot,cbAddData,dwOffset);
			dwOffset  += cbAddData;

			/* Add extents */
			cbAddData = sizeof(float);
			temp 	 = -M_PI;		// Xll
			hDdeData  = DdeAddData(hDdeData,(LPBYTE)&temp,cbAddData,dwOffset);
			dwOffset  += cbAddData;
			temp 	 = -M_PI;		// Yll
			hDdeData  = DdeAddData(hDdeData,(LPBYTE)&temp,cbAddData,dwOffset);
			dwOffset  += cbAddData;
			temp 	 = M_PI;		// Xur
			hDdeData  = DdeAddData(hDdeData,(LPBYTE)&temp,cbAddData,dwOffset);
			dwOffset  += cbAddData;
			temp 	 = M_PI;		// Yur
			hDdeData  = DdeAddData(hDdeData,(LPBYTE)&temp,cbAddData,dwOffset);
			dwOffset  += cbAddData;

			/* Add data. */

			/* Some versions of DDEML exhibit problems when adding very large
			   chunks of data. Here we have set the maximum to MAX_ADD (defined above)
			   and loop until Z array is exhausted. */
			dwSize = (NX+1)*(NY+1)*sizeof(float);
			j = 0;
			while( dwSize >	0 ) {
				cbAddData = min(MAX_ADD,dwSize);
				hDdeData  = DdeAddData(hDdeData,(LPBYTE)&Z[j],cbAddData,dwOffset);
				if( !hDdeData ) {
					sprintf(szText,"Error code for DdeAddData=%#x",DdeGetLastError(idInst));
					DdeFreeStringHandle(idInst,hszItem);
					DdeDisconnect(hConvDPLOT);
					free(Z);
					MessageBox(hwnd,szText,"DDE Error",MB_OK | MB_ICONEXCLAMATION);
					return TRUE;
				}
				dwOffset += cbAddData;
				dwSize -= cbAddData;
				j += cbAddData/sizeof(float);
			}
			cbAddData = DPlot.SizeofExtraInfo;
			hDdeData  = DdeAddData(hDdeData,szExec,cbAddData,dwOffset);
			dwOffset  += cbAddData;

			if( DdeClientTransaction( (LPBYTE)hDdeData,
			   -1,
			   hConvDPLOT,
			   hszItem,
			   cfFormat,
			   XTYP_POKE,
			   TimeOut,
			   NULL 		  ) == NULL ) {
				sprintf(szText,"Error code for DdeClientTransaction=%#x",DdeGetLastError(idInst));
				MessageBox(hwnd,szText,"DDE Error",MB_OK | MB_ICONEXCLAMATION);
			}
			DdeFreeStringHandle(idInst,hszItem);
			DdeDisconnect(hConvDPLOT);
			free(Z);
			t2 = GetTickCount();
			sprintf(szText,"Elapsed time = %d milliseconds",t2-t1);
			SetDlgItemText(hwnd,DLG_TIME,szText);
			return TRUE;

		case DLG_EXITBUTTON:
			EndDialog( hwnd, TRUE );
			return TRUE;

		case IDCANCEL:
			EndDialog( hwnd, TRUE );
			return TRUE;

		default:
			return FALSE;

		}

	case WM_DESTROY:
		DdeUninitialize(idInst);
		return FALSE;

	case WM_PAINT:
		if(hbmp) DrawBitmapToCtrl(hwnd,DLG_BMPFRAME);
	}
	
	return FALSE;
	
} /* PlotOptionsDlgProc */

// extern VOID	   Plot();

/*
* WinMain - initialization, message loop
*/
int PASCAL WinMain( HINSTANCE this_inst, HINSTANCE prev_inst, LPSTR cmdline,
				   int cmdshow )
{
	hbmp = (HBITMAP)NULL;
	DialogBox( this_inst,"DPLOTDEMO", NULL, PlotOptionsDlgProc );
	if(hbmp) DeleteObject(hbmp);
	return 0;
	
} /* WinMain */


