/* Structure used to send data to DPLOT via DDE */


#define DPLOT_DDE_VERSION 2
#define DPLOT_VERSION 2

typedef struct tagDPLOT
{
    DWORD Version;           // version number of this structure
    DWORD hwnd;              // handle of client application window
                             // (Use DWORD rather than HWND)
    DWORD DataFormat;        // XY pairs, DX and Y, etc.
    DWORD MaxCurves;         // maximum number of curves (must be <= 20)
                             // = NX for DataFormat = DATA_3D
                             //   ignore for DataFormat = DATA_3DR
    DWORD MaxPoints;         // maximum number of points/curve
                             // = NY for DataFormat = DATA_3D
                             // = 3 * number of points for DATA_3DR
    DWORD NumCurves;         // actual number of curves, always 1 for
                             //   DATA_3D or DATA_3DR
    DWORD Scale;             // scaling code (Linear, Log, etc.)
    float LegendX;           // left coord of legend, expressed as a ratio
                             //    of plot size (0->1)
    float LegendY;           // top coord of legend
    DWORD NP[20];            // actual number of points in each curve;
                             //   cannot exceed MaxPoints.
	                         // For DATA_3DR files, return number of nodes in NP[0]
    DWORD LineType[20];      // line types (see codes below)
    DWORD SymbolType[20];    // symbol types (see codes below)
    DWORD SizeofExtraInfo;   // Extra information following X,Y data
    char  Legend[21][40];    // Legend[0] is the caption for the legend.
                             // Legend[n] is the legend for the n'th curve.
    char  Label[20][5];      // Strings displayed beside the last data point
                             //   in a curve.
							 // NOTE: Though DPlot 1.7.1.1 and later allow up
	                         //       to 40 characters for these labels, to
	                         //       avoid confusion this structure is unchanged
	                         //       (labels still limited to 5 characters).
	                         //       If you need longer labels, use the
	                         //       CurveLabel command.
    char  Title[3][80];      // Three title lines.
    char  XAxis[80];         // X Axis label.
    char  YAxis[80];         // Y Axis label.
} DPLOT;

// DPLOT file status specifiers for "Plot DPLOT File" message

#define DPLOT_FILE_KEEP 0
#define DPLOT_FILE_DELETE 1

// Scaling codes:

#define SCALE_LINEARX_LINEARY     1
#define SCALE_LINEARX_LOGY        2
#define SCALE_LOGX_LINEARY        3
#define SCALE_LOGX_LOGY           4
#define SCALE_TRIPARTITE          5
#define SCALE_LINEARX_PROBABILITY 6
#define SCALE_GRAINSIZE_DIST      7
#define SCALE_POLAR               8
#define SCALE_BARCHART            9
#define SCALE_LOGX_PROBABILITY   10
#define SCALE_PROBX_LINEARY      11
#define SCALE_PROBX_LOGY         12
#define SCALE_PROBX_PROBY        13
#define SCALE_TRIANGLE           14

// Unit-specific scaling codes ... combine with above values using OR operator

#define UNITS_DEFAULT                   0
#define UNITS_TRIPARTITE_INCHES      UNITS_DEFAULT   // Velocity (Y) in inches/sec
#define UNITS_TRIPARTITE_FEET        0x00000100L     //                 feet/sec
#define UNITS_TRIPARTITE_MILLIMETERS 0x00000200L     //                 mm/sec
#define UNITS_TRIPARTITE_CENTIMETERS 0x00000300L     //                 cm/sec
#define UNITS_TRIPARTITE_METERS      0x00000400L     //                 meters/sec
#define UNITS_GRAINSIZE_MILLIMETERS  UNITS_DEFAULT   // Grain sizes (X) in mm
#define UNITS_GRAINSIZE_INCHES       0x00000100L     //                    inches
#define UNITS_POLAR_RADIANS          UNITS_DEFAULT   // Rotation (X) in radians
#define UNITS_POLAR_DEGREES          0x00000100L     //                 degrees
#define UNITS_USERDEFINED            0x00007F00L

// Data organization following DPLOT structure:

#define DATA_XYXY         0
#define DATA_DXY          1
#define DATA_XYYY         2
#define DATA_3D           3     // Z values on a rectangular grid
#define DATA_3DR          4     // Random X,Y,Z values

// Line styles:

#define LINESTYLE_NONE        0
#define LINESTYLE_SOLID       1
#define LINESTYLE_LONGDASH    2
#define LINESTYLE_DOTTED      3
#define LINESTYLE_DASHDOT     4
#define LINESTYLE_MEDDASH     5
#define LINESTYLE_DASHDOTDOT  6

// Symbol styles
#define SYMBOLSTYLE_NONE         0
#define SYMBOLSTYLE_DOT          1
#define SYMBOLSTYLE_CROSS        2
#define SYMBOLSTYLE_ASTERISK     3
#define SYMBOLSTYLE_X            4
#define SYMBOLSTYLE_SQUARE       5
#define SYMBOLSTYLE_DIAMOND      6
#define SYMBOLSTYLE_TRIANGLE     7
#define SYMBOLSTYLE_OCTAGON      8
#define SYMBOLSTYLE_ITRIANGLE    9
#define SYMBOLSTYLE_HEXAGON     10
#define SYMBOLSTYLE_PENTAGON    11
#define SYMBOLSTYLE_STAR        12
#define SYMBOLSTYLE_FILL        0x00000100L

// DPLOT File Types

#define FILETYPE_DPLOT           1
#define FILETYPE_XANDY           2
#define FILETYPE_CONSTANTDX      3
#define FILETYPE_MULTIPLEY       4
#define FILETYPE_UNFORMATTED     5
#define FILETYPE_BENDIX          6     // Pacific digitizer, DNA/Bendix format
#define FILETYPE_WESOLD          7     // Pacific digitizer, pre-1993
#define FILETYPE_WESNEW          8     // Pacific digitizer, 1993+
#define FILETYPE_NICOLETTIME     9     // Nicolet oscilloscope time domain
#define FILETYPE_HDAS           10     // Hardened Data Acquistion System

#define FILETYPE_3DR            11     // ASCII file with random X,Y,Z triplets

// Plug-in types

#define PLUGIN_REPLACEDATA  1L    // Plug-in operates on and replaces existing data
                                  //    in the same document.
#define PLUGIN_NEWDATA      2K    // Plug-in operates on existing data (or not) and
                                  //    returns one or more data sets (curves) for the
                                  //    same document.
#define PLUGIN_NEWDOCUMENT  3L    // Plug-in operates on existing data (or not) and
                                  //    returns an entirely new document.

// Plug-in return codes
#define PLUGIN_BADARRAYSIZE -2L		// Current array sizes insufficient. Required size
									// should be returned in MaxCurves and MaxPoints
									// members of DPLOT structure
#define PLUGIN_FAILURE      -1L		// General purpose "didn't work"
#define PLUGIN_ABORT         0L		// Operation aborted by user
#define PLUGIN_SUCCESS       1L


